package com.example.ellegadodepintia.exploradoresDePintia.model


import android.graphics.drawable.AnimationDrawable
import android.os.Handler
import android.os.Looper
import android.widget.ImageView
import android.widget.TextView
import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.model.GameState.context
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.Efecto
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoInvestigarEnemigo
import com.example.ellegadodepintia.exploradoresDePintia.model.eventoFactory.EventoCombate
import com.example.ellegadodepintia.exploradoresDePintia.model.habilidad.efectoHabilidad.EfectoHabilidad

object GestorCombate {

    lateinit var evento : EventoCombate
    private var turnoJugador = true

    fun cargarAnimacionesAtaqueProtagonista() {
        val imagenProtagonista = context!!.findViewById<ImageView>(R.id.imagenProtagonista)
        imagenProtagonista?.setImageResource(R.drawable.protagonista_ataque_animation)
        val ataqueAnimation = imagenProtagonista?.drawable as? AnimationDrawable
        ataqueAnimation?.start()

        Handler(Looper.getMainLooper()).postDelayed({
            imagenProtagonista?.setImageResource(R.drawable.protagonista_idle_animation)
            val idleAnimation = imagenProtagonista?.drawable as? AnimationDrawable
            idleAnimation?.start()
        }, 750)
    }

    fun cargarAnimacionesMuerteProtagonista() {
        val imagenProtagonista = context!!.findViewById<ImageView>(R.id.imagenProtagonista)
        imagenProtagonista?.setImageResource(R.drawable.protagonista_die_animation)
        val muerteAnimation = imagenProtagonista?.drawable as? AnimationDrawable
        muerteAnimation?.start()

        val handler = Handler(Looper.getMainLooper())
        handler.postDelayed({
            muerteAnimation!!.stop()
            // Cambiar al último frame manualmente
            imagenProtagonista.setImageDrawable(muerteAnimation.getFrame(muerteAnimation.numberOfFrames - 1))
        }, 900)
    }

    fun cargarAnimacionesAtaqueEnemigo() {
        val imagenEvento = context!!.findViewById<ImageView>(R.id.imagenEvento)
        imagenEvento?.setImageResource(evento.imagenAtaque)
        val ataqueEnemigo = imagenEvento?.drawable as? AnimationDrawable
        ataqueEnemigo?.start()

        val ataqueDuracion = ataqueEnemigo?.let {
            (0 until it.numberOfFrames).sumOf { i -> it.getDuration(i) }
        } ?: 0

        Handler(Looper.getMainLooper()).postDelayed({
            imagenEvento?.setImageResource(evento.imagen)
            val idleAnimation = imagenEvento?.drawable as? AnimationDrawable
            idleAnimation?.start()
        }, ataqueDuracion.toLong())
    }

    fun calcularAtaque(ataqueBase: Int): Int {
        val ataqueFinal: Int
        val resultadoTirada = (0..100).random()
        ataqueFinal = if (resultadoTirada <= 8) {
            ataqueBase * 2
        } else {
            ataqueBase
        }
        return ataqueFinal
    }

    private fun comenzarAccionEnemigo() {
        if (evento.resistenciaActual > 0 && !evento.finalizado) {
            Handler(Looper.getMainLooper()).postDelayed({
                accionEnemigo()
            }, 800)
        } else {
            evento.actualizarFinalizado(true)
            turnoJugador = true
            "¡Enhorabuena has acabado con el monstruo!".also {
                context!!.findViewById<TextView>(R.id.textoResultado).text = it
            }
        }
    }

    fun accionProtagonista(efecto: Efecto): String {
        val mensaje: String
        if (turnoJugador) {
            turnoJugador = false
            restarTurnoBuffos()
            mensaje = efecto.ejecutar()
            if (efecto !is EfectoInvestigarEnemigo) {
                comenzarAccionEnemigo()
            }
        } else {
            mensaje = "No es tu turno"
        }
        return mensaje
    }

    fun habilidadProtagonista(nombre: String, efecto: EfectoHabilidad, energia: Int) {
        val textoResultado = context!!.findViewById<TextView>(R.id.textoResultado)
        if(GameState.jugador.atributos[Atributo.Energia]!! >= energia){
            turnoJugador = false
            "${textoResultado.text} \n\n Se ha utilizado $nombre".also { textoResultado.text = it }
            restarTurnoBuffos()
            efecto.ejecutar()
            GameState.jugador.actualizarAtributo(Atributo.Energia, -energia)
            comenzarAccionEnemigo()
        }else{
            "${textoResultado.text} \n\n Energía insuficiente".also { textoResultado.text = it }
        }
    }

    private fun restarTurnoBuffos(){
        for (buffo in GameState.jugador.buffos) {
                buffo.pasarTurno()
        }
    }


    private fun accionEnemigo(){
            val textoResultado = context!!.findViewById<TextView>(R.id.textoResultado)
            val efectoAtaque = evento.acciones.random().ejecutar()
            if(GameState.jugador.atributos[Atributo.Vida]!! <= 0){
                Handler(Looper.getMainLooper()).postDelayed({
                    turnoJugador = true
                }, 2000)
            } else {
                turnoJugador = true
                "${textoResultado.text} \n\n $efectoAtaque".also { textoResultado.text = it }
            }
    }

}